@@ -2,10 +2,12 @@
 # The model is parameterized similarly to tree models, but the returned value is a time series of diversity (number of clades) over time
 simulate_diversification_model = function(	times,									# times at which to calculate diversities, in ascending order
 											parameters				= list(), 		# named list of model parameters. For entries and default values see the main function body below
-											start_time				= NULL,			# If reverse==TRUE, then this is the end time of the tree (>=times[NT]), otherwise this is the beginning time of the tree (<=times[1]). If NULL, this is set to times[1] or times[NT], depending on reverse.
-											start_diversity			= 1,			# diversity of extant clades at start_time. If reverse==TRUE, then this is the visible diversity after rarefaction.
-											coalescent				= FALSE,		# (boolean) if true, the diversities at any time are replaced by the diversities that would remain in a coalescent tree (i.e. only including ancestors of extant tips)
+											start_time				= NULL,			# Beginning (earliest) time of the simulation (<=times[1]). If NULL, this is set to times[1]. If reverse==FALSE, the tree is simulated forwards from this time point.
+											final_time				= NULL,			# Final (latest) time of the simulation (>=times[NT]). If NULL, this is set to times[NT]. If reverse==TRUE, the tree is simulated backwards from this time point.
+											start_diversity			= 1,			# diversity of extant clades at start_time. Only relevant if reverse==FALSE.
+											final_diversity			= NULL,			# diversity of extant clades at final_time (prior to any rarefaction or collapsing at some resolution). Only relevant if reverse==TRUE.
 											reverse					= FALSE,		# (boolean) if true, then the tree model is integrated in backward time direction. In that case, start_diversity is interpreted as the true diversity at times.back()
+											include_coalescent		= FALSE,		# (boolean) if true, the coalescent diversities are also calculated (in addition to the total diversities)
 											include_event_rates		= FALSE,		# (boolean) include birth & death rates in returned values
 											include_Nevents			= FALSE,		# (boolean) include an estimate of the total birth (speciation) and death (extinction) events during the simulation
 											max_runtime				= NULL){		# (numeric) max allowed runtime in seconds. If NULL or <=0, this option is ignored
@@ -13,8 +15,13 @@ simulate_diversification_model = function(	times,									# times at which to ca
 	
 	# basic error checking
 	if((!is.null(start_time)) && (!reverse) && (start_time>times[1])) stop(sprintf("start_time must be equal to or smaller than the first requested time point (got start_time=%g, first requested time point = %g)",start_time,times[1]))
-	if((!is.null(start_time)) && reverse && (start_time<times[NT])) stop(sprintf("start_time must be equal to or larger than the last requested time point (got start_time=%g, last requested time point = %g)",start_time,times[NT]))
-	if(is.null(start_time)) start_time = (if(reverse) times[NT] else times[1]);
+	if((!is.null(final_time)) && reverse && (final_time<times[NT])) stop(sprintf("final_time must be equal to or larger than the last requested time point (got final_time=%.10g, last requested time point = %.10g)",final_time,times[NT]))
+	if(is.null(start_time)) start_time = times[1];
+	if(is.null(final_time)) final_time = times[NT];
+	if(reverse && is.null(final_diversity)) stop(sprintf("Missing final_diversity, needed for simulating tree in reverse"))
+	if((!reverse) && is.null(start_diversity)) stop(sprintf("Missing start_diversity, needed for simulating tree forwards"))
+	if(is.null(start_diversity)) start_diversity = NaN; # not relevant, but assign a dummy value needed for the CPP function
+	if(is.null(final_diversity)) final_diversity = NaN; # not relevant, but assign a dummy value needed for the CPP function
 							
 	# set default model parameters
 	if(is.null(parameters$birth_rate_intercept)) 	parameters$birth_rate_intercept = 0;
@@ -23,6 +30,7 @@ simulate_diversification_model = function(	times,									# times at which to ca
 	if(is.null(parameters$death_rate_intercept)) 	parameters$death_rate_intercept = 0;
 	if(is.null(parameters$death_rate_factor))		parameters$death_rate_factor = 0;
 	if(is.null(parameters$death_rate_exponent)) 	parameters$death_rate_exponent = 1;
+	if(is.null(parameters$resolution)) 				parameters$resolution = 0;
 	if(is.null(parameters$rarefaction)) 			parameters$rarefaction = 1;
 						
 	# run simulation		
@@ -32,13 +40,16 @@ simulate_diversification_model = function(	times,									# times at which to ca
 																death_rate_intercept 		= parameters$death_rate_intercept,
 																death_rate_factor 			= parameters$death_rate_factor,
 																death_rate_exponent 		= parameters$death_rate_exponent,
+																resolution					= parameters$resolution,
 																rarefaction					= parameters$rarefaction,
 																Nsplits 					= 2,
 																times 						= times,
 																start_time					= start_time,
+																final_time					= final_time,
 																start_diversity				= start_diversity,
+																final_diversity				= final_diversity,
 																reverse						= reverse,
-																coalescent					= coalescent,
+																include_coalescent			= include_coalescent,
 																include_probabilities		= TRUE,
 																include_birth_rates			= include_event_rates,
 																include_death_rates			= include_event_rates,
@@ -61,14 +72,15 @@ simulate_diversification_model = function(	times,									# times at which to ca
 		Pdiscovery = Prepresentation/Psurvival
 	}
 
-	return(list(success				= simulation$success,
-				diversities			= simulation$diversities,
-				Psurvival			= Psurvival,
-				Pdiscovery			= Pdiscovery,
-				Prepresentation		= Prepresentation,
-				birth_rates			= (if(include_event_rates) simulation$birth_rates else NULL),
-				death_rates			= (if(include_event_rates) simulation$death_rates else NULL),
-				Nbirths				= simulation$Nbirths,
-				Ndeaths				= simulation$Ndeaths,
-				error				= (if(!simulation$success) simulation$error else NULL)));
+	return(list(success					= simulation$success,
+				coalescent_diversities	= (if(include_coalescent) simulation$coalescent_diversities else NULL),
+				total_diversities		= simulation$total_diversities,
+				Psurvival				= Psurvival,
+				Pdiscovery				= Pdiscovery,
+				Prepresentation			= Prepresentation,
+				birth_rates				= (if(include_event_rates) simulation$birth_rates else NULL),
+				death_rates				= (if(include_event_rates) simulation$death_rates else NULL),
+				Nbirths					= simulation$Nbirths,
+				Ndeaths					= simulation$Ndeaths,
+				error					= (if(!simulation$success) simulation$error else NULL)));
 }
